/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
**+
** NAME:
**      xppp
**
** PURPOSE:
**      A Motif interface to handle an IP connection
**
**      User options in xppp.h 
**
** USAGE:
**      xppp
**  
** MODIFICATION HISTORY:
**
**      v1.0: 1997 Apr, RSM: Fixed problem when the status pixmap could not
**            be allocated.  Formerly, xppp (incorrectly) would exit if it 
**            could not allocate the pixmap; now it just continues 
**            without using the status pixmap.
**
**      v0.9: 1996 Oct, written, Robert S. Mallozzi
**            mallozzir@cspar.uah.edu
**            (http://cspar.uah.edu/~mallozzir/)
**-
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

 
#include "xppp.h"

#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <unistd.h>
#include <ctype.h>
#include <time.h>

#include <Xm/Xm.h>

#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/PushB.h>
#include <Xm/LabelG.h>
#include <Xm/SeparatoG.h>
#include <Xm/Text.h>
 
#include <X11/xpm.h>
#include "xppp_icon.xpm"



/* Global variables * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

 /* Main application */
XtAppContext app;

/* Widget ids */
Widget toplevel, text_output, password_dialog;
Widget PostDialog();

/* Functions */
void exit(), OpenPipe(), GetFromChild(), wprint();
void Start_ppp(), Stop_ppp(), PingHost();
void HelpCB(), PhoneSelectionDialogCB();
void CheckPasswd(), DestroyPasswd();
void TerminateOk();
void PostMenu();
void QuitCB();

/* File descriptor for the pipe to the shell scripts and process ids */
int pipefd[2], pid;

/* User password */
char *passwd;

/* Parameters passed to the shell scripts, e.g. passwd */
char *pass_command_1, *pass_command_2; 

Boolean LINK_UP;

/* Parameters for the pixmap status label */
void         update(); 
XtIntervalId timer;
Widget       status_label;

/* Parameters for the time label */
void         update_time();
time_t       start_time;
XtIntervalId counter;
Widget       time_label;

/* End global variables * * * * * * * * * * * * * * * * * * * * * * * * * * * */




/*   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *   *   */
main(int argc, char *argv[])
{
    Widget   sepg;
    Widget menubar, menu, help_widget;
    XmString file, quit, help, about;
    Widget rowcol, form;
    Widget init_ppp, term_ppp, ping;
    Arg args[10];
    int n, j;

    Pixmap pixm;
    char current_pixm[MAX_STRING];
    char button_title[MAX_STRING];
         
    /* For reading the host data file */
    Widget popup, wid, cascade;
    FILE *host_file;
    char *file_name = PPP_DATA_DIR;
    char *data_file_name;
    char name[MAX_STRING], address[MAX_STRING], newline;
    int i;
    
    typedef struct {
        char *name;
        char *address;
    } ListItem;
    
    ListItem host_items[MAX_HOSTS];

    static String fallbackResources[] =
                  {
                  "*text_output.rows: 6",
                  "*text_output.columns: 80",
                  NULL
                  }; 

    sprintf (current_pixm, "%s/%s", PIXMAP_DIR, PIXMAP_STOP);

    XtSetLanguageProc (NULL, NULL, NULL);

    /* Initialize the toolkit and create the toplevel shell widget 
    **  The "Xppp" specifies the applications default file
    */
    toplevel = XtVaAppInitialize (&app, "Xppp", NULL, 0,
        &argc, argv, fallbackResources, NULL);

    /* Specify the icon */         
    XpmCreatePixmapFromData(XtDisplay (toplevel),
        RootWindowOfScreen (XtScreen (toplevel)),
        icon, &pixm, NULL, NULL); 
      
    XtVaSetValues (toplevel,
        XmNiconPixmap, pixm,
        NULL);
/*
    XFreePixmap(XtDisplay (toplevel), pixm);
*/
         
    rowcol = XtVaCreateWidget ("rowcol", 
        xmRowColumnWidgetClass, toplevel, NULL);
 
    /* A simple menu bar */
    file = XmStringCreateLocalized ("File");
    help = XmStringCreateLocalized ("Help");
    menubar = XmVaCreateSimpleMenuBar(rowcol, "menubar",
        XmVaCASCADEBUTTON, file, 'F',  
        XmVaCASCADEBUTTON, help, 'H',
        NULL);
    
    /* HELP button goes on the RHS of the menubar */
    if (help_widget = XtNameToWidget (menubar, "button_1"))
        XtVaSetValues (menubar, XmNmenuHelpWidget, help_widget, NULL);

    /* Add a callback for QUIT under FILE menu */
    quit = XmStringCreateLocalized ("Quit");
    menu = XmVaCreateSimplePulldownMenu (menubar, "file_menu", 0, QuitCB,
        XmVaPUSHBUTTON, quit, 'Q', NULL, NULL, 
        NULL);

    /* Add callbacks for HELP */  
    help = XmStringCreateLocalized("Help");
    about = XmStringCreateLocalized ("About");
    XmVaCreateSimplePulldownMenu (menubar, "help_menu", 1, HelpCB,
        XmVaPUSHBUTTON, help, 'H', NULL, NULL, 
        XmVaSEPARATOR, 
        XmVaPUSHBUTTON, about, 'A', NULL, NULL,
        NULL);
 
    XmStringFree (file);
    XmStringFree (quit);
    XmStringFree (help);        
    XmStringFree (about);
    XtManageChild (menubar);    
   
    /* Add a ScrolledText widget to hold the status data */
    n = 0;
    /* Size now set in app-defaults file
    ** XtSetArg(args[n], XmNrows,                  6); n++;
    ** XtSetArg(args[n], XmNcolumns,               80); n++;
    */
    XtSetArg(args[n], XmNeditable,              False); n++;
    XtSetArg(args[n], XmNeditMode,              XmMULTI_LINE_EDIT); n++;
    XtSetArg(args[n], XmNwordWrap,              True); n++;
    XtSetArg(args[n], XmNscrollHorizontal,      False); n++;
    XtSetArg(args[n], XmNscrollVertical,        True); n++;
    XtSetArg(args[n], XmNcursorPositionVisible, False); n++;
    text_output = XmCreateScrolledText (rowcol, "text_output", args, n);
    XtManageChild (text_output);
        
    form = XtVaCreateManagedWidget ("form",
        xmFormWidgetClass, rowcol, 
        XmNfractionBase,   19,
        NULL);
        
    strcpy (button_title, "Initiate ");
    strcat (button_title, PROTOCOL);
    strcat (button_title, " Link");
    init_ppp = XtVaCreateManagedWidget (button_title, 
        xmPushButtonWidgetClass, form,        
        XmNleftAttachment,       XmATTACH_POSITION,
        XmNleftPosition,         0,
        XmNrightAttachment,      XmATTACH_POSITION,
        XmNrightPosition,        6,
        NULL);                
    XtAddCallback (init_ppp, XmNactivateCallback, 
        Start_ppp, NULL);

    strcpy (button_title, "Terminate ");
    strcat (button_title, PROTOCOL);
    strcat (button_title, " Link");
    term_ppp = XtVaCreateManagedWidget (button_title, 
        xmPushButtonWidgetClass, form, 
        XmNleftAttachment,       XmATTACH_WIDGET,
        XmNleftWidget,           init_ppp,        
        XmNrightAttachment,      XmATTACH_POSITION,
        XmNrightPosition,        12,
        NULL);
    strcat (button_title, "?");
    XtAddCallback (term_ppp, XmNactivateCallback, 
        Stop_ppp, button_title);

    ping = XtVaCreateManagedWidget ("Ping Host", 
        xmPushButtonWidgetClass, form, 
        XmNleftAttachment,       XmATTACH_WIDGET,
        XmNleftWidget,           term_ppp,        
        XmNrightAttachment,      XmATTACH_POSITION,
        XmNrightPosition,        18,
        NULL);
    /* Callback for ping added after host data is read */

    /* Add a pixmap label, which is a green or red button signifying if
    ** the connection is up or down
    */
    pixm = XmGetPixmap( XtScreen(toplevel),
        current_pixm,
        BlackPixelOfScreen(XtScreen(toplevel)),
        WhitePixelOfScreen(XtScreen(toplevel)));
    if (pixm == XmUNSPECIFIED_PIXMAP) {
       printf("xppp: Could not allocate pixmap\n");    
    }
    
    status_label = XtVaCreateManagedWidget ("label", 
        xmLabelGadgetClass,  form, 
        XmNleftAttachment,   XmATTACH_WIDGET,
        XmNleftWidget,       ping,
        XmNrightAttachment,  XmATTACH_POSITION,
        XmNrightPosition,    19,
        XmNlabelType,        XmPIXMAP,
        XmNlabelPixmap,      pixm,
        NULL);

    /* Add a popup menu that contains the hosts defined in 
    ** PPP_DATA_DIR/HOST_FILE
    */
    popup = XmCreatePopupMenu (ping, "popup", NULL, 0);
    XtVaSetValues (popup, 
        XmNtearOffModel, XmTEAR_OFF_ENABLED, 
        NULL);
    
    /* Add the menu title and a separator */
    wid = XtVaCreateManagedWidget ("Ping Hosts",
        xmLabelGadgetClass, popup, 
        NULL);
    wid = XtVaCreateManagedWidget ("_sep1",
        xmSeparatorGadgetClass, popup, 
        NULL);

    /* Read the host data */
    if ((data_file_name = (char *) malloc (strlen (HOST_FILE))) == NULL) {
       perror ("xppp");
       exit (1);
    }
    strcpy (data_file_name, HOST_FILE);
    
    if ((file_name = (char *) malloc (strlen (PPP_DATA_DIR) + 1 + 
       strlen (data_file_name))) == NULL) {
       perror ("xppp");
       exit (1);
    }
    strcpy (file_name, PPP_DATA_DIR);

    strcat (file_name, "/");
    strcat (file_name, data_file_name);

    if ((host_file = fopen (file_name, "r")) == NULL) {
       perror ("xppp");      
       exit (1);
    }

    i = 0;
    while (fscanf (host_file, "%s%s%c", name, address, &newline) != EOF) {
 
          if ((host_items[i].name = 
             (char *) malloc (strlen (name))) == NULL) {
             perror ("xppp");
             exit (1);
          }
          if ((host_items[i].address = 
             (char *) malloc (strlen (address))) == NULL) {
             perror ("xppp");
             exit (1);
          }   

          strcpy (host_items[i].name, name);          
          strcpy (host_items[i].address, address);
 
          wid = XtVaCreateManagedWidget (host_items[i].name,
              xmPushButtonWidgetClass, popup, 
              NULL);
              
          /* Pass the IP address to the callback */
          XtAddCallback (wid, XmNactivateCallback,      
              PingHost, host_items[i].address); 
 
          i++;
    }
    fclose (host_file);
 
    /* Add the event handler for the menu */
    XtAddEventHandler (ping, ButtonPressMask, False, PostMenu, popup);
   
    /* Still maintain the callback for the "Ping Host" button.
    **  Ping the first host listed in PPP_DATA_DIR/HOST_FILE
    **  by default.
    */ 
    XtAddCallback (ping, XmNactivateCallback,
        PingHost, host_items[0].address);
  
    sepg = XtVaCreateManagedWidget ("sepg",
        xmSeparatorGadgetClass, rowcol,
        NULL);        
  
    time_label = XtVaCreateManagedWidget ("time_label", 
        xmTextWidgetClass,        rowcol,
        XmNvalue,                 "00:00:00",
        XmNcursorPositionVisible, False,
        XmNeditable,              False,
        NULL);
 
    XtManageChild (rowcol); 

    /* Add a timer to update the pixmap status label 
    ** (update every 10 seconds)
    */
    timer = XtAppAddTimeOut(app, 10000, (XtPointer) update, NULL);

    LINK_UP = FALSE;          

    XtRealizeWidget (toplevel);
    XtAppMainLoop (app);
}



/*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  */
void 
update(XtPointer client_data)
{

    FILE *check_ppp;
    char current_pixm[MAX_STRING];
    Pixmap old, pixm;
  
    /* Done with old pixmap, destroy it */
    XtVaGetValues (status_label, 
        XmNlabelPixmap, &old, 
        NULL);
    XmDestroyPixmap (XtScreen (toplevel), old);

    if ((check_ppp = fopen (LINK_FILE, "r")) == NULL) {
 
       LINK_UP = FALSE;
       
       sprintf (current_pixm, "%s/%s", PIXMAP_DIR, PIXMAP_STOP);

       if (counter) 
          XtRemoveTimeOut (counter);

       XmTextSetString (time_label, "00:00:00");            
 
    }
    else {

       if (LINK_UP == FALSE)
          time (&start_time);
          
       LINK_UP = TRUE;
       
       sprintf (current_pixm, "%s/%s", PIXMAP_DIR, PIXMAP_GO);    
      
       /* Close the file if the fopen succeeded */
       fclose (check_ppp);
 
       /* Add a timer for the connect time counter label 
       ** (update every 1 second)
       */
       if (! counter) 
       counter = XtAppAddTimeOut(app, 1000, (XtPointer) update_time, NULL);        
    }
    
    pixm = XmGetPixmap( XtScreen(toplevel),
        current_pixm,
        BlackPixelOfScreen(XtScreen(toplevel)),
        WhitePixelOfScreen(XtScreen(toplevel)));
    if (pixm != XmUNSPECIFIED_PIXMAP) {
    
       XtVaSetValues (status_label,
           XmNlabelPixmap, pixm,
           NULL);
    }
    
    /* Now add the timer again so that update() is called recursively */
    timer = XtAppAddTimeOut(app, 10000, (XtPointer) update, NULL);

}


/*  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  *  */
void 
update_time(XtPointer client_data)
{
    time_t       current_time;
    int          hrs, min, sec;
    char         hrs_s[3], min_s[3], sec_s[3];
    char         time_s[10];
    double       elapsed_time;
        
    if (LINK_UP == FALSE)
       return;
       
    time (&current_time);    
    
    elapsed_time = difftime( current_time, start_time );
     
    hrs = (int) elapsed_time / 3600;
    min = ((int) elapsed_time % 3600) / 60;
    sec = ((int) elapsed_time % 3600) % 60;
    
    if (hrs < 10)
       sprintf (hrs_s, "0%d", hrs);
    else
       sprintf (hrs_s, "%2d", hrs);

    if (min < 10)
       sprintf (min_s, "0%d", min);
    else
       sprintf (min_s, "%2d", min);       

    if (sec < 10)
       sprintf (sec_s, "0%d", sec);
    else
       sprintf (sec_s, "%2d", sec); 

    sprintf (time_s, "%s:%s:%s", hrs_s, min_s, sec_s);

    XmTextSetString (time_label, time_s);            

    /* Now add the timer again so that update_time() is called recursively */
    counter = XtAppAddTimeOut(app, 1000, (XtPointer) update_time, NULL);        


}



void
QuitCB(Widget widget, XtPointer client_data, XtPointer call_data)
{ 
    exit (0);
    
}   
